// Morph-O-Mat - Call BkgtoMorf for sequence of objects from list
// By Arnie Cachelin Copyright 2000, NewTek, Inc.

// This script automatically creates endomorphs (MORF VMaps) from a series of objects.
// The objects must have matching point order, since this is how foreground and 
// background points are correlated.

// The format of the morf list file is simple. The first line is the name of the object to 
// use as the base object. Endomorphs will be added to this mesh. The second line is the name 
// that the final object will be saved as. After that, the lines alternate, first the file name 
// for the morph target, then the name for the new endomorph. All object file names are 
// prepended with the file directory specified by the user. For example The following lines 
/* could be a morph list file:
BaseTexturedHead.lwo
Goodhead.lwo
morfs/head1.lwo
Mouth.Sad
morfs/head2.lwo
Mouth.Happy
morfs/head3.lwo
Mouth.Angry
morfs/head_crazy17.lwo
Nose.Busted
morfs/head4.lwo
Nose.Petit

*/
@warnings
@version 1.2

mFile;
filePath = "Objects/";
listName = "morflist.txt";

main
{
    reqbegin("Morph-O-Mat: Multiple EndoMorph Machine");
    reqsize(387,112);
    c1 = ctlfilename("Morph List","string");
    ctlposition(c1,37,16);
    c2 = ctlstring("Object Directory","string");
    ctlposition(c2,59,48);
	setvalue(c1,listName);
	setvalue(c2,filePath);
    if(reqpost())
    {
        listName = getvalue(c1);
        filePath = getvalue(c2);
    }

	mfile = File(listName,"r");
	if(mfile==nil)
    {
        error("Cannot get Morph List File");
		return;
    }
    if(mfile.isOpen() == false)
    {
        error("Cannot open Morph List File");
        return;
    }
	num = (mfile.linecount()/2) + 1;
	//str = "Processing "+num+" Endomorphs";
    //moninit(num + 1,str);
    //monstep();  

	baseMorf = mfile.read(); // 1st object is base
	saveName = mfile.read(); // first name is save name for new endomorph object
	lname = filePath+baseMorf;
	load(lname);
	fg = lyrfg();
	empty = lyremptybg();   // get empty background layers
	bg = 2;//empty;
	while(mfile.eof()==false)
	{
		baseName = mfile.read();
		if(baseName=="")
			return;
		morfName = mfile.read();
		if(morfName=="")
			return;
		lname = filePath+baseName;
		load(lname);
		//monstep();  

        selmode("USER");
        selpoint("SET");
        copy();
        close();
        setobject(baseMorf);
        lyrsetfg(bg); 
        paste();
        lyrsetfg(fg); 
        lyrsetbg(bg); 

        if(cmdseq("Bkg-To-MORF",morfName) != 0)
        {
            error("Error executing Bkg-To-Morph plug-in.  Terminating.");
            return;
        }
        lyrsetfg(bg); 
        cut();
    }
    lyrsetfg(fg);
	lname = filePath+saveName;
	save(lname);
}
