//-----------------------------------------
// LWMacro.ls - This Master Class Script
// will record a user's events and save
// them out as a macro

@version 2.0
@warnings
@script master

//declare an array to hold the commands and it's counter

var commandBuffer;
var commandMember = 0;

//declare a variable to hold the script mode

modeSet = 0;

//declare a variable to hold the macro file path

macro;
macroFile;

//declare a variable to hold the addMacro state

addMacro = 0; 

create
{
    //set the title in the server pane

    setdesc("LW Macro Recorder Beta 3");
}

flags
{
    //set flags so that we monitor all Layout events

    return(SCENE, OBJECTS, EFFECTS);
}

process: event, command
{
    if(modeSet == 1)
    {    
        //increment the array counters

        commandMember++;

        //assign the last executed command to the buffer

        commandBuffer[commandMember] = command;
     }
}

options
{
    //Make sure that the requester isn't already open

    if(reqisopen())
        reqend();
    {    
        //create the requester

        reqbegin("LW Macro Recorder");

        reqsize(200,100);

        //set up and initialize the controls

        c1 = ctlstate("Record",modeSet,50,"scriptMode");   
        c2 = ctlbutton("Reset",50,"ClearBuffer");
        c3 = ctlbutton("Output File",75,"SaveFile");
        c4 = ctlcheckbox("Add to Command List",addMacro);
               
        ctlrefresh(c1,"ModeRefresh");
        ctlrefresh(c4,"AddRefresh");

        ctlposition(c1,10,10);
        ctlposition(c2,65,10);
        ctlposition(c3,10,45);
        ctlposition(c4,90,45);
        
        reqopen();
    }
}

createMacro: macro
{ 
        //declare the variables required to perform the operation

        var bufferSize, commandSize;
        var rawMacroLine;
        var macroLine;

        //get the size of the command buffer

        bufferSize = size(commandBuffer);
 
        //add the pragma directives, main process and the opening brace

        macro.writeln("@version 2.0");
        macro.writeln("@warnings");
        macro.writeln("@script generic");
        macro.writeln("");
        macro.writeln("generic");
        macro.writeln("{");

        //step through the command buffer and write the macro lines

        for(x = 1;x <= bufferSize;x++)
        {
            //parse the command line so we can reconstruct it

            rawMacroLine = parse(" ",commandBuffer[x]);
    
            commandSize = size(rawMacroLine);
    
            //if the command doesn't require any arguments strip the second, "(null)", element

            if(rawMacroLine[1] == "AddPartigon")
                macroLine = "CommandInput(\"AddPartigon\");";
            else if((strleft(rawMacroLine[1],8)) == "Generic_")
            {
                //if the command is a generic plugin construct
                //a command to pass it as an argument to
                //CommandInput();

                macroLine += "CommandInput(\"";
                macroLine += rawMacroLine[1];
                macroLine += "\");";
            }
            else if((commandSize == 2) && (rawMacroLine[2] == "(null)"))
            {
                macroLine = rawMacroLine[1] + "();";
            }
            else
            {
                //construct the command line from the available elements

                macroLine = rawMacroLine[1] + "(";

                if(((strleft(macroLine,3)) == "Add") || ((strleft(macroLine,7)) == "Replace") ||
((strleft(macroLine,4)) == "Load") || ((strleft(macroLine,4)) == "Save") ||
((strleft(macroLine,7)) == "Content") || (macroLine == "Rename("))
                {  
                    //if the command is one that requires a single
                    //string argument add the required quotation marks 
                    //and the closing paranthesis and semi-colon
                  
                    macroLine += "\"";
                    macroLine += rawMacroLine[2];
                    macroLine += "\");";
                }
                else if(rawMacroLine[1] == "ApplyServer")
                {
                    //if the command is "ApplyServer" add quotations
                    //around both string arguments.  This is a 
                    //special case because it is the only command
                    //that requires two string arguments.

                    for(z = 2;z <= commandSize;z++)
                    {
                        macroLine += "\"";
                        macroLine += rawMacroLine[z];
                        macroLine += "\"";
    
                        if(z != commandSize)
                            macroLine += ",";
                        else
                        {
                            //if it's the last command add the closing
                            //paranthesis and semi-colon
                            macroLine += ");";
                        }
                    }
                }
                else
                {
                    //if the command doesn't require string arguments
                    //then simply separate the arguments with a comma

                    for(y = 2;y <= commandSize;y++)
                    {                   
                        macroLine += rawMacroLine[y];

                        if(y != commandSize)
                            macroLine += ",";
                        else
                        {
                            //if it's the last command add the closing
                            //paranthesis and semi-colon

                            macroLine += ");";
                        }
                    }
                }
            }
            
            //write the new line to the macro file

            macro.writeln(macroLine);
        }

        //once all the commands have been written add the closing brace

        macro.writeln("}");
        macro.close();
}

//This UDF is used by the ctlrefresh() function to update the mode button

ModeRefresh: val
{
    modeSet = val;
}

//This UDF is used by the ctlrefresh() function to update the Add Macro button

AddRefresh: val
{
    addMacro = val;
}

//This UDF presents the user with a file requester to choose a file to save to

SaveFile
{
    if(commandBuffer)
    {   
        if((macroFile = getfile("Macro File...","*.ls")) == nil)
            return;

        //assign the selected file to the object agent

        if((macro = File(macroFile,"w")) == nil)
        {
            error("Could not create macro file!");
            return;
        }

        //run the createMacro() UDF to create the macro file

        createMacro(macro);

        if(addMacro)
        {
            AddPlugins(macroFile);
        
            info("<br>Macro Added.  It is now accessible as a command in the Edit Menus and Edit Keys windows");
        }
    }
    else
    {
        info("<br>There are no commands to create a macro with.  Set to record mode and perform some operations");    
    }
}

//This UDF is called when the user clicks the 'Reset' button

ClearBuffer
{
    if(commandBuffer)
    {
        commandBuffer[] = nil;
    }
}

//This UDF is used by the ctlstate() button 'Record'

scriptMode: val
{
    modeSet = val;
}
